import os
from jsonextended import edict

from aiida_crystal17.tests import TEST_DIR
from aiida_crystal17.gulp.potentials.reaxff import (
    read_reaxff_file, write_lammps, PotentialWriterReaxff)


def test_parse():

    fpath = os.path.join(TEST_DIR, 'gulp_input_files', 'FeCrOSCH.reaxff')
    data = read_reaxff_file(fpath)

    # print(data)
    exp_data = {}

    exp_data["tolerances"] = {
        "anglemin": 0.001,
        "angleprod": 0.000001,
        "hbondmin": 0.01,
        "hbonddist": 7.5,
        "torsionprod": 0.000000001
    }
    # NB: torsionprod needs to be lower to get comparable energy to lammps,
    # but then won't optimize
    exp_data[
        'descript'] = "Reactive MD-force field: Cr/O/Fe/S/C/H force field 2014"
    exp_data['params'] = {
        'Overcoordination 1': 50.0,
        'Torsion/BO': 5.7796,
        'Overcoordination 2': 9.5469,
        'Lower Taper-radius': 0.0,
        'Valency overcoordination 1': 0.6991,
        'Valency overcoordination 2': 50.0,
        'Conjugation': 2.1645,
        'Double bond/angle': 6.929,
        'Triple bond stabilisation': 4.6,
        'Valency undercoordination': 33.8667,
        'Valency angle/lone pair': 6.0891,
        'Triple bond stabilisation 2': 6.8702,
        'Triple bond stabilisation 1': 1.7224,
        'C2-correction': 60.485,
        'Not used 8': 0.0,
        'Not used 4': -1.2327,
        'Not used 5': 0.5,
        'Not used 6': 20.0,
        'Upper Taper-radius': 10.0,
        'Not used 1': 2.8793,
        'Not used 2': 6.1431,
        'Undercoordination 3': 13.3056,
        'Torsion overcoordination 1': 10.0,
        'Torsion overcoordination 2': 1.9487,
        'vdWaals shielding': 1.5591,
        'Valency/lone pair': 1.8512,
        'Valency angle 2': 2.0384,
        'Double bond/angle: overcoord 2': 3.9954,
        'Double bond/angle: overcoord 1': 0.3989,
        'Valency angle 1': 1.0563,
        'Triple bond stabilization energy': -70.5044,
        'bond order cutoff': 0.1,
        'Valency angle conjugation 1': 26.5405,
        'Valency angle conjugation 2': 2.1365,
        'Valency angle conjugation 3': 2.6962,
        'Not used 7': 5.0,
        'Undercoordination 1': 1.0588,
        'Undercoordination 2': 12.1176,
        'Not used 3': -2.4837
    }

    exp_data['species'] = {
        'reaxff1_over4':
        [-2.8983, -19.4571, -3.55, -16.0573, -15.7363, -11.1953, -11.0],
        'reaxff1_over3':
        [13.5366, 0.0003, 0.0021, 0.759, 15.323, 0.759, 0.669],
        'reaxff1_over2': [8.9539, 3.0408, 3.5027, 25.343, 12.0, 25.343, 8.741],
        'reaxff1_over1':
        [34.9289, 2.4197, 0.764, 10.126, 22.1978, 10.126, 13.364],
        'dummy8': [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
        'reaxff1_under': [34.935, 0.0, 37.5, 0.0, 30.0, 0.0, 0.0],
        'dummy2': [0.0, 1.0, 2.0, 0.0, 2.0, 0.0, 0.0],
        'dummy3': [202.5551, 61.6606, 59.0626, 102.1, 65.0, 102.1, -2.37],
        'dummy1': [79.5548, 121.125, 116.0768, 18.3725, 112.1416, 18.319, 0.0],
        'dummy6': [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
        'dummy7': [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
        'dummy4': [0.8563, 1.0698, 0.9745, 0.8563, 0.9745, 0.8563, 0.9745],
        'dummy5': [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
        'dummy11': [1.0564, 1.0338, 1.0493, 1.0338, 1.0338, 1.0338, 1.0338],
        'reaxff_gamma': [0.9, 0.8203, 1.0898, 0.4744, 0.753, 0.5639, 0.01],
        'reaxff_chi': [
            5.9666, 3.7248, 8.5, 1.7785, 6.5745, 1.4546, 5.00009999
        ],
        'reaxff1_radii3': [1.2114, -0.1, 0.9049, -1.2, 1.0, -1.2, -0.1],
        'reaxff1_radii2': [
            1.1341, -0.1, 1.0548, -1.6836, 1.6468, -1.6836, -0.1
        ],
        'reaxff1_radii1': [1.3817, 0.893, 1.245, 1.9029, 1.8328, 1.8921, -0.1],
        'reaxff1_angle1': [2.5, 4.2733, 2.9, 2.6997, 2.8802, 2.6997, 2.7466],
        'reaxff1_lonepair2': [0.0, 0.0, 0.4056, 0.0, 3.4994, 0.0, 0.0],
        'reaxff1_morse4': [
            2.1346, 33.2894, 13.8449, 2.6084, 4.9055, 2.8702, 2.5
        ],
        'symbol': ['C', 'H', 'O', 'Fe', 'S', 'Cr', 'X'],
        'reaxff1_angle2': [
            2.9663, 2.8793, 2.9225, 2.5791, 2.8793, 2.5791, 2.8793
        ],
        'reaxff1_valence4': [4.0, 1.0, 4.0, 3.0, 4.0, 6.0, 4.0],
        'reaxff1_valence1': [4.0, 1.0, 2.0, 3.0, 2.0, 6.0, 2.0],
        'reaxff1_valence2': [4.0, 1.0, 4.0, 6.0, 6.2998, 6.0, 2.0],
        'reaxff1_valence3': [4.0, 1.0, 6.0, 3.0, 6.0, 6.0, 6.0],
        'reaxff_mu': [7.0, 9.6093, 8.3122, 8.6281, 9.0, 8.95, 9999.0],
        'idx': [1, 2, 3, 4, 5, 6, 0],
        'reaxff1_morse1': [9.7559, 8.223, 9.73, 10.8548, 9.0, 10.3177, 10.0],
        'reaxff1_morse2': [0.1838, 0.093, 0.1, 0.1181, 0.3236, 0.2336, 0.0],
        'reaxff1_morse3': [1.8903, 1.355, 2.389, 2.099, 1.8815, 2.3712, 2.0],
        'mass': [12.0, 1.008, 15.999, 55.845, 32.06, 51.9962, 1.008],
        'reaxff1_lonepair1': [0.0, 0.0, 2.0, 0.0, 2.0, 0.0, 2.0]
    }

    exp_data['bonds'] = {
        'reaxff2_bo3': [
            -0.1, 1.0, 1.0, -0.2288, 1.0, -0.1055, -0.35, 1.0, -0.35, -0.2,
            -0.1016, 1.0, -0.3341, -0.1838, -0.1677, -0.35, -0.35, -0.2339,
            -0.3, -0.2, -0.35
        ],
        'reaxff2_bo2': [
            6.7268, 6.9136, 6.0552, 4.8734, 4.279, 5.5, 4.1708, 5.4965, 7.1208,
            6.8013, 5.5813, 5.0961, 6.9512, 6.3424, 6.144, 8.0, 4.8803, 5.666,
            5.3349, 6.122, 8.0
        ],
        'reaxff2_bo1': [
            -0.0777, -0.05, -0.079, -0.1363, -0.092, -0.1225, -0.1512, -0.1543,
            -0.0629, -0.0512, -0.1579, -0.0762, -0.0909, -0.0758, -0.1131,
            -0.2, -0.1989, -0.1166, -0.1098, -0.0925, -0.2
        ],
        'reaxff2_over': [
            0.4147, 0.7652, 0.73, 1.0, 0.5626, 0.6051, 0.01, 0.1766, 0.0852,
            0.2682, 0.1958, 0.387, 0.05, 0.1769, 0.0999, 0.5, 0.6591, 0.0835,
            0.01, 0.0106, 0.5
        ],
        'reaxff2_bo7': [
            1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 0.0, 0.0, 1.0, 1.0, 1.0,
            0.0, 1.0, 1.0, 1.0, 1.0, 0.0, 0.0, 1.0
        ],
        'reaxff2_bo6': [
            37.6117, 6.0, 6.0, 10.5036, 6.0, 29.7503, 36.0, 6.0, 36.0, 16.0,
            18.9617, 6.0, 22.1005, 16.0, 17.8574, 36.0, 36.0, 36.0, 16.0, 16.0,
            36.0
        ],
        'reaxff2_bo5': [
            -0.455, 0.0, 0.0, -0.2351, 0.0, -0.1, -0.3, 0.0, -0.3, -0.2,
            -0.5211, 0.0, -0.2406, -0.339, -0.4781, -0.3, -0.3, -0.3891, -0.3,
            -0.1, -0.3
        ],
        'reaxff2_bo4': [
            9.1628, 0.0, 0.0, 7.025, 0.0, 9.0, 15.0, 0.0, 15.0, 15.0, 13.875,
            0.0, 7.9877, 15.0, 8.2557, 15.0, 15.0, 12.1279, 16.0, 15.0, 15.0
        ],
        'reaxff2_bond5': [
            0.459, 5.229, 6.25, 0.4475, 1.115, 0.3451, 1.0648, 0.5673, 1.0,
            0.6294, 2.0, 11.836, 0.6528, 0.28, 0.2799, 0.3, 0.5793, 0.29,
            0.0842, 0.2209, 0.3
        ],
        'reaxff2_bond4': [
            -0.7738, -0.6083, -0.46, -0.3739, -0.5725, 0.2506, 0.6663, 0.4668,
            0.1301, 0.2931, -0.5652, -0.6562, 0.4728, -0.4815, -0.9993, 1.0,
            0.2305, 0.6879, -1.0, 0.9374, 1.0
        ],
        'reaxff2_bond3': [
            78.0, 0.0, 0.0, 60.8077, 0.0, 50.8293, 0.0, 0.0, 0.0, 0.0, 55.2528,
            0.0, 40.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0
        ],
        'reaxff2_bond2': [
            99.1897, 0.0, 0.0, 82.6772, 0.0, 145.0, 0.0, 0.0, 0.0, 0.0,
            90.5383, 0.0, 202.9813, 0.0, 69.1367, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0
        ],
        'reaxff2_bond1': [
            158.2004, 169.476, 153.3934, 164.4303, 160.0, 142.2858, 109.5214,
            78.2669, 67.5128, 41.4611, 192.1462, 188.3744, 107.2917, 75.528,
            86.8868, 0.0, 114.0666, 88.6258, 57.5947, 41.4444, 0.0
        ],
        'reaxff2_bo8': [
            1.0, 0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 0.0, 0.0, 0.0, 1.0, 1.0, 1.0,
            0.0, 1.0, 1.0, 1.0, 1.0, 0.0, 0.0, 1.0
        ],
        'idx2':
        [1, 2, 2, 3, 3, 3, 4, 4, 4, 4, 5, 5, 5, 5, 5, 6, 6, 6, 6, 6, 6],
        'idx1':
        [1, 1, 2, 1, 2, 3, 1, 2, 3, 4, 1, 2, 3, 4, 5, 2, 3, 5, 6, 4, 1],
        'dummy1': [
            1.0, 1.0, 1.0, 1.0, 0.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0,
            1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0
        ],
        'reaxff2_bo9': [
            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0
        ]
    }
    exp_data['off-diagonals'] = {
        'reaxff2_morse1': [
            0.1239, 0.1345, 0.0283, 0.4204, 0.02, 0.1, 0.3314, 0.102, 0.2832,
            0.0854, 0.1, 0.0582, 0.2523, 0.3236, 0.1
        ],
        'reaxff2_morse3': [
            1.4004, 1.8422, 1.2885, 1.49, 1.9451, 1.8, 1.7976, 1.7528, 1.8196,
            1.7474, 1.5, 1.7, 1.9844, 2.167, 1.5
        ],
        'reaxff2_morse2': [
            9.8467, 9.7725, 10.919, 11.0144, 10.8595, 9.1989, 10.5605, 9.6276,
            10.2295, 12.7924, 11.0, 11.6513, 10.1396, 10.1297, 11.0
        ],
        'reaxff2_morse5': [
            -1.0, 1.1576, -1.0, -1.0, -1.0, -1.0, 1.4, -1.0, 1.4557, -1.0,
            -1.0, -1.0, -1.0, -1.0, -1.0
        ],
        'reaxff2_morse4': [
            1.121, 1.2835, 0.9215, 1.4071, 1.4157, 1.705, 1.6918, 1.3714,
            1.4502, 1.9838, -1.0, 1.5924, 1.8938, 1.9025, -1.0
        ],
        'reaxff2_morse6': [
            -1.0, 1.0637, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0,
            -1.0, -1.0, -1.0, -1.0
        ],
        'idx2': [2, 3, 3, 4, 4, 4, 5, 5, 5, 5, 6, 6, 6, 6, 6],
        'idx1': [1, 1, 2, 1, 2, 3, 1, 2, 3, 4, 2, 3, 5, 4, 1]
    }
    exp_data['hbonds'] = {
        'reaxff3_hbond1': [2.12, 2.5, 2.5, 2.5],
        'reaxff3_hbond3': [1.45, 1.45, 1.45, 1.45],
        'reaxff3_hbond2': [-3.58, -1.0, -1.0, -2.0],
        'reaxff3_hbond4': [19.5, 19.5, 19.5, 19.5],
        'idx3': [3, 5, 3, 5],
        'idx1': [3, 3, 5, 5],
        'idx2': [2, 2, 2, 2]
    }
    exp_data['torsions'] = {
        'dummy2': [
            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0
        ],
        'reaxff4_torsion5': [
            -1.6, -2.1802, -2.2677, -1.0, -1.0, -1.0, -1.0, -1.0, -2.9807,
            -3.0, -2.8, -3.0, -3.0, -2.0, -2.0, -2.9498, 0.0, -0.9972, 0.0,
            -0.9, 0.0, 0.0, 0.0, 0.0, -1.0, -2.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
            0.0, 0.0, 0.0, 0.0, 0.0, 0.0
        ],
        'reaxff4_torsion4': [
            -6.3507, -4.9581, -4.8923, -2.5, -4.6995, -4.9622, -2.5, -6.1481,
            -2.7733, -4.2274, -3.6877, -4.5172, -5.5315, -9.0, -4.4848,
            -5.4085, -2.6, -2.5004, -6.6539, -2.5, 0.0, 0.0, -2.5415, -4.0,
            -5.1903, -9.0, -5.0, 0.0, -8.0, -8.0, -10.0, -10.0, -8.0, -6.0,
            -6.0, -8.0, -8.0, -8.0
        ],
        'reaxff4_torsion3': [
            0.0288, 0.2945, 0.4539, -0.5249, 0.7914, 0.6461, -0.0057, -0.3139,
            0.6165, 0.7582, 1.0, -0.0964, -1.0, 1.5, -1.5, 0.0268, 0.2, 0.9706,
            0.5, 0.1, 0.0, 0.0, 0.02, 0.3, 1.133, 0.1601, 0.0, 0.0, 0.01, 0.0,
            0.2237, 0.5, 0.5, 0.5, 0.5, 0.0, 0.0, 0.0
        ],
        'reaxff4_torsion2': [
            34.7453, 29.2131, 31.2081, 20.7787, 19.8113, 16.6853, 19.6316,
            80.0, 31.0191, 70.8145, 80.0, 78.0747, 70.3345, 20.0, 80.0, 0.1181,
            5.0, 0.5219, 43.184, 1.0, 0.0, 0.0, 0.1, 50.0, 25.415, 20.1851,
            0.0, 0.0, 71.9719, 50.0, 2.5, 50.0, 50.0, 90.0, 90.0, 50.0, 50.0,
            50.0
        ],
        'reaxff4_torsion1': [
            -0.25, -0.25, -0.25, 1.2799, 1.9159, -1.4477, 0.4816, 1.2044, -2.5,
            -2.4875, -1.4383, -1.139, -2.5, -0.1583, 0.0002, -2.5, 0.1995,
            0.4118, 0.1, 0.1, 0.0, 0.0, 0.0, 0.0, 0.5511, -0.0002, 0.0, 0.0,
            2.4661, 0.0, 2.5, 0.5, 0.5, 0.25, 0.25, 0.0, 0.0, 0.0
        ],
        'idx3': [
            1, 1, 1, 1, 1, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 2, 2,
            3, 1, 3, 3, 4, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5
        ],
        'idx2': [
            1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3, 3, 3, 3, 3, 3, 1, 2,
            2, 1, 3, 1, 1, 2, 5, 5, 3, 1, 3, 5, 5, 5, 5, 5
        ],
        'idx1': [
            1, 1, 2, 1, 2, 3, 1, 1, 2, 2, 2, 3, 3, 3, 1, 1, 2, 1, 2, 3, 0, 0,
            0, 0, 0, 1, 2, 0, 5, 2, 2, 0, 0, 3, 3, 1, 1, 2
        ],
        'dummy1': [
            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0
        ],
        'idx4': [
            1, 2, 2, 3, 3, 3, 1, 2, 1, 2, 3, 1, 2, 3, 1, 2, 2, 3, 3, 3, 0, 0,
            0, 0, 0, 3, 4, 0, 5, 2, 3, 0, 0, 5, 3, 1, 2, 2
        ]
    }
    exp_data['angles'] = {
        'reaxff3_penalty': [
            6.2933, 0.0, 0.0, 0.0, 0.0, 0.0, 58.6562, -50.0, 0.0, 68.1072,
            50.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
            0.0, 0.0, 0.0, 0.0, 0.0, 0.0
        ],
        'reaxff3_conj': [
            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -25.3063, 0.0, 0.0, 0.0, 0.0,
            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.01, 0.0, 0.0, 0.0,
            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.5, 0.5, 0.0, 0.0,
            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
            0.0, 0.0, 0.0, 0.0, 0.0, 0.0005, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
            0.0, 0.0, 0.0, 0.0
        ],
        'reaxff3_angle3': [
            0.7606, 6.2481, 3.7312, 7.735, 6.0, 5.8635, 2.6122, 2.4177, 0.7205,
            1.7397, 0.9953, 5.273, 4.4734, 2.0, 2.272, 3.0, 3.0, 2.89, 0.3169,
            2.0, 7.9341, 3.0111, 2.2825, 1.0568, 7.9607, 0.6773, 0.9167, 0.25,
            2.2066, 6.6776, 2.3543, 0.1151, 0.05, 3.2567, 0.1898, 2.9929,
            1.7223, 0.05, 8.0001, 3.7715, 0.5302, 1.0424, 6.7228, 3.1104,
            0.5449, 2.5588, 7.0, 0.9881, 7.0, 0.7741, 2.1707, 3.693, 1.9918,
            6.0, 6.0, 2.0, 1.3736, 0.7752, 0.1076, 4.9579, 0.824, 1.0, 2.8121,
            2.9681, 1.3919, 3.0, 1.178, 4.2281, 0.3698, 2.0, 1.0, 1.0, 1.0,
            2.0, 1.0, 2.0, 2.0, 2.995, 2.0, 0.3584, 1.0, 1.0, 3.3161, 1.3988,
            2.0221, 2.9881, 2.8285, 2.9247, 2.7567, 1.3736, 2.0, 1.8851,
            1.0871, 0.9049, 3.0, 1.0, 2.0, 2.0, 1.0, 2.0, 1.0, 2.0, 2.0, 2.0,
            0.9493, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 2.0, 2.0, 1.0,
            2.0, 2.0, 1.0
        ],
        'reaxff3_angle2': [
            30.7029, 14.5234, 25.2202, 3.411, 0.0, 27.9213, 7.8968, 44.2852,
            42.551, 32.2258, 30.4554, 16.3141, 13.1217, 50.0, 9.8453, 25.0,
            8.5744, 15.0, 42.298, 30.0, 1.0246, 34.6443, 0.01, 0.01, 10.4428,
            4.0632, 32.1638, 0.1451, 0.01, 42.4716, 12.6123, 19.3103, 49.6772,
            0.01, 11.2599, 14.8127, 32.633, 0.0583, 5.7742, 15.579, 42.2756,
            8.0885, 27.2223, 15.6798, 11.8933, 14.1666, 5.0, 37.5859, 29.5117,
            23.4174, 30.0701, 38.299, 19.1501, 0.25, 1.0, 7.5, 7.5083, 9.2519,
            16.6274, 7.4569, 16.4865, 10.0, 8.1452, 5.5458, 7.0973, 12.9742,
            26.4957, 30.0, 17.2148, 20.0, 0.0, 0.0, 0.0, 20.0, 0.0, 20.0, 20.0,
            23.5125, 20.0, 19.2597, 0.0, 0.0, 40.0, 0.26, 26.6099, 20.0,
            10.459, 19.75, 15.3276, 7.5083, 20.0, 16.3822, 30.0, 21.4421, 30.0,
            0.0, 20.0, 20.0, 0.0, 20.0, 0.0, 20.0, 20.0, 20.0, 34.5234, 0.0,
            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 20.0, 20.0, 0.0, 20.0,
            20.0, 0.0
        ],
        'reaxff3_angle1': [
            59.0573, 65.7758, 70.2607, 0.0, 0.0, 0.0, 53.9517, 76.9627,
            72.6199, 81.9029, 80.7324, 65.0, 70.1101, 75.6935, 85.8, 0.0, 0.0,
            0.0, 0.1, 74.879, 47.9341, 33.2812, 20.3683, 0.0, 0.0, 48.4128,
            2.6539, 42.714, 0.0, 90.0, 54.69, 38.2755, 26.0012, 0.0, 38.5594,
            63.074, 73.6721, 76.5431, 69.4895, 84.2345, 90.0601, 66.1035,
            71.4462, 92.671, 43.6595, 99.1897, 89.391, 83.5231, 79.9791,
            83.1032, 60.2631, 55.9402, 35.9099, 0.0, 0.0, 0.0, 0.2, 100.0,
            33.4003, 77.0475, 84.8837, 0.0, 29.3282, 4.8461, 89.9073, 63.9878,
            54.9212, 44.9499, 51.2416, 70.0, 0.0, 0.0, 0.0, 70.0, 0.0, 70.0,
            70.0, 87.7418, 70.0, 50.5809, 0.0, 0.0, 76.3546, 61.0461, 44.3862,
            41.9243, 19.4072, 83.6415, 89.6617, 0.2, 70.0, 93.8709, 60.3275,
            67.5806, 89.6657, 0.0, 70.0, 70.0, 0.0, 70.0, 0.0, 70.0, 70.0,
            70.0, 63.7706, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 70.0,
            70.0, 0.0, 70.0, 70.0, 0.0
        ],
        'reaxff3_angle5': [
            0.718, 0.5665, 0.005, 0.0, 0.0, 0.0, 3.0, 1.6334, 2.9294, 0.9888,
            3.0, 0.4448, 0.8433, 1.0, 2.8635, 1.0, 0.0, 0.0, 1.1069, 2.0334,
            2.8853, 0.1701, 0.766, 1.8595, 2.3717, 2.2274, 0.024, 0.0851,
            1.9789, 2.456, 2.0, 0.7569, 1.1589, 2.0582, 0.1904, 0.7552, 1.0221,
            0.4968, 1.7794, 1.3066, 0.3707, 0.7355, 0.005, 0.3458, 0.005,
            0.3542, 1.005, 1.4725, 0.005, 1.2168, 1.3323, 2.2673, 0.9342, 0.0,
            0.0, 0.0, 0.0412, 0.1221, 0.0825, 0.7548, 0.5428, 0.25, 0.2287,
            1.0491, 0.821, 0.5058, 2.0, 1.6672, 1.5585, 1.0, 1.0, 1.0, 1.0,
            1.0, 1.0, 1.0, 1.0, 0.5199, 1.0, 0.2153, 1.0, 1.0, 1.9473, 0.9535,
            1.9212, 1.2678, 1.0074, 1.9437, 1.6031, 0.0412, 1.0, 0.321, 1.3106,
            0.7789, 2.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 2.4647,
            1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0,
            1.0, 1.0
        ],
        'reaxff3_angle4': [
            1.1244, 1.6255, 2.75, 1.04, 1.04, 1.04, 1.0338, 2.7392, 1.3096,
            1.7777, 1.0783, 1.4077, 3.0, 1.168, 1.58, 1.04, 1.0421, 2.8774,
            2.3466, 1.0928, 1.0, 1.051, 1.3788, 3.6142, 1.197, 1.8605, 1.1158,
            2.8955, 1.4466, 1.6221, 1.2513, 2.3113, 1.0, 1.3513, 1.4041,
            1.3634, 1.4351, 2.2792, 2.7889, 1.627, 1.0071, 3.0, 2.6454, 2.3207,
            1.9326, 2.599, 1.5, 1.0641, 1.2255, 2.7365, 1.0192, 1.0, 2.7883,
            1.04, 1.04, 1.04, 1.8149, 2.2142, 1.0, 2.3345, 1.1398, 1.5, 1.9412,
            1.4836, 2.9921, 3.0, 2.2398, 1.2, 1.5755, 1.05, 1.05, 1.05, 1.05,
            1.05, 1.05, 1.05, 1.0, 1.6571, 1.0, 1.6345, 1.05, 1.05, 1.0, 3.0,
            3.0, 2.6481, 2.3949, 1.0348, 1.0781, 1.8149, 1.0, 1.9357, 1.6977,
            1.2, 1.2, 1.05, 1.05, 1.05, 1.05, 1.05, 1.05, 1.05, 1.05, 1.05,
            1.7747, 1.05, 1.05, 1.05, 1.05, 1.05, 1.05, 1.05, 1.05, 1.05, 1.05,
            1.05, 1.05, 1.05, 1.05, 1.05
        ],
        'idx3': [
            1, 2, 2, 1, 2, 2, 3, 3, 1, 3, 3, 3, 2, 3, 2, 3, 3, 3, 1, 4, 4, 4,
            2, 4, 4, 4, 4, 2, 4, 4, 4, 3, 4, 4, 3, 4, 4, 3, 4, 5, 2, 5, 5, 1,
            5, 2, 5, 3, 3, 5, 5, 5, 5, 5, 5, 5, 5, 4, 5, 5, 4, 5, 5, 5, 4, 3,
            6, 6, 6, 6, 3, 6, 6, 2, 6, 6, 5, 6, 6, 6, 6, 6, 5, 6, 4, 6, 5, 6,
            6, 6, 5, 6, 5, 4, 5, 6, 2, 6, 6, 1, 6, 6, 3, 6, 6, 4, 6, 6, 4, 6,
            6, 6, 2, 6, 6, 5, 6, 5, 6, 6
        ],
        'idx1': [
            1, 1, 2, 1, 1, 2, 1, 3, 1, 1, 3, 2, 1, 2, 2, 1, 2, 3, 1, 1, 1, 4,
            2, 2, 4, 2, 2, 1, 1, 1, 3, 1, 2, 3, 2, 4, 3, 3, 3, 5, 2, 2, 1, 1,
            2, 1, 1, 3, 1, 1, 3, 3, 2, 1, 3, 5, 5, 4, 4, 4, 2, 4, 3, 4, 3, 3,
            6, 3, 3, 2, 2, 3, 2, 2, 6, 2, 5, 6, 5, 5, 2, 1, 5, 4, 3, 3, 3, 3,
            5, 5, 4, 4, 1, 1, 4, 1, 1, 2, 1, 1, 6, 1, 1, 3, 1, 1, 1, 4, 2, 2,
            4, 1, 1, 2, 1, 1, 5, 2, 2, 5
        ],
        'idx2': [
            1, 1, 1, 2, 2, 2, 1, 1, 3, 3, 3, 1, 3, 3, 3, 2, 2, 2, 4, 1, 4, 1,
            4, 2, 2, 4, 1, 4, 2, 3, 1, 4, 3, 2, 4, 3, 3, 4, 4, 5, 5, 5, 1, 5,
            1, 5, 5, 5, 5, 3, 3, 5, 3, 2, 2, 2, 4, 5, 4, 5, 5, 2, 4, 3, 5, 6,
            3, 3, 6, 3, 6, 2, 6, 6, 2, 2, 6, 5, 5, 6, 5, 5, 3, 3, 6, 4, 6, 5,
            3, 4, 6, 5, 4, 5, 1, 2, 6, 1, 6, 6, 1, 1, 6, 1, 3, 6, 4, 1, 6, 4,
            2, 2, 6, 1, 5, 6, 1, 6, 5, 2
        ]
    }

    assert edict.diff(data, exp_data, np_allclose=True) == {}


def test_write_gulp():
    fpath = os.path.join(TEST_DIR, 'gulp_input_files', 'FeCrOSCH.reaxff')
    data = read_reaxff_file(fpath)

    expected = """#
#  ReaxFF force field
#
#  Original paper:
#
#  A.C.T. van Duin, S. Dasgupta, F. Lorant and W.A. Goddard III,
#  J. Phys. Chem. A, 105, 9396-9409 (2001)
#
#  Parameters description:
#
#  Reactive MD-force field: Cr/O/Fe/S/C/H force field 2014#  (Filtered by: ['Fe', 'S'])
#
#  Cutoffs for VDW & Coulomb terms
#
reaxFFvdwcutoff      10.0000
reaxFFqcutoff        10.0000
#
#  Bond order threshold - check anglemin as this is cutof2 given in control file
#
reaxFFtol  0.0010000000 0.0010000000 0.0000010000 0.0100000000 7.5000000000 0.0000000010
#
#  Species independent parameters 
#
reaxff0_bond        50.000000     9.546900
reaxff0_over        50.000000     0.699100     1.058800    12.117600    13.305600
reaxff0_valence     33.866700     1.851200     1.056300     2.038400
reaxff0_penalty      6.929000     0.398900     3.995400
reaxff0_torsion      5.779600    10.000000     1.948700     2.164500
reaxff0_vdw          1.559100
reaxff0_lonepair     6.089100
#
#  Species parameters 
#
reaxff1_radii 
Fe core   1.9029  -1.6836  -1.2000 
S  core   1.8328   1.6468   0.0000 
reaxff1_valence 
Fe core   3.0000   6.0000   3.0000   3.0000 
S  core   2.0000   6.2998   6.0000   4.0000 
reaxff1_over 
Fe core  10.1260  25.3430   0.7590 -16.0573 
S  core  22.1978  12.0000  15.3230 -15.7363 
reaxff1_under kcal 
Fe core   0.0000 
S  core  30.0000 
reaxff1_lonepair kcal 
Fe core   0.0000   0.0000 
S  core   2.0000   3.4994 
reaxff1_angle 
Fe core   2.6997   2.5791 
S  core   2.8802   2.8793 
reaxff1_morse kcal 
Fe core  10.8548   0.1181   2.0990   2.6084 
S  core   9.0000   0.3236   1.8815   4.9055 
#
# Element parameters 
#
reaxff_chi 
Fe core   1.7785 
S  core   6.5745 
reaxff_mu 
Fe core   8.6281 
S  core   9.0000 
reaxff_gamma 
Fe core   0.4744 
S  core   0.7530 
#
# Bond parameters 
#
reaxff2_bo over bo13 
S  core S  core  -0.1131   6.1440  -0.1677   8.2557   0.0000  17.8574 
reaxff2_bo 
Fe core Fe core  -0.0512   6.8013  -0.2000  15.0000   0.0000  16.0000 
Fe core S  core  -0.0758   6.3424  -0.1838  15.0000   0.0000  16.0000 
reaxff2_bond kcal 
Fe core Fe core  41.4611   0.0000   0.0000   0.2931   0.6294 
Fe core S  core  75.5280   0.0000   0.0000  -0.4815   0.2800 
S  core S  core  86.8868  69.1367   0.0000  -0.9993   0.2799 
reaxff2_over 
Fe core Fe core   0.2682 
Fe core S  core   0.1769 
S  core S  core   0.0999 
reaxff2_morse kcal 
Fe core S  core   0.0854  12.7924   1.7474   1.9838  -1.0000  -1.0000 
#
# Angle parameters 
#
reaxff3_angle kcal 
S  core S  core S  core  84.2345  15.5790   3.7715   1.6270   1.3066 
Fe core S  core S  core   0.2000   7.5083   1.3736   1.8149   0.0412 
S  core Fe core Fe core 100.0000   9.2519   0.7752   2.2142   0.1221 
Fe core Fe core S  core  33.4003  16.6274   0.1076   1.0000   0.0825 
S  core Fe core S  core  77.0475   7.4569   4.9579   2.3345   0.7548 
reaxff3_penalty kcal 
S  core S  core S  core   0.0000 
Fe core S  core S  core   0.0000 
S  core Fe core Fe core   0.0000 
Fe core Fe core S  core   0.0000 
S  core Fe core S  core   0.0000 
#
# Hydrogen bond parameters 
#
#
# Torsion parameters 
#
reaxff4_torsion kcal 
S  core S  core S  core S  core   2.4661  71.9719   0.0100  -8.0000   0.0000 
"""  # noqa: W291

    string = PotentialWriterReaxff().create_string(
        data, species_filter=['Fe', 'S'])
    assert string == expected


def test_write_lammps():

    fpath = os.path.join(TEST_DIR, 'gulp_input_files', 'FeCrOSCH.reaxff')
    data = read_reaxff_file(fpath)

    expected2 = """Reactive MD-force field: Cr/O/Fe/S/C/H force field 2014
39 ! Number of general parameters
50.0000 ! Overcoordination 1
9.5469 ! Overcoordination 2
26.5405 ! Valency angle conjugation 1
1.7224 ! Triple bond stabilisation 1
6.8702 ! Triple bond stabilisation 2
60.4850 ! C2-correction
1.0588 ! Undercoordination 1
4.6000 ! Triple bond stabilisation
12.1176 ! Undercoordination 2
13.3056 ! Undercoordination 3
-70.5044 ! Triple bond stabilization energy
0.0000 ! Lower Taper-radius
10.0000 ! Upper Taper-radius
2.8793 ! Not used 1
33.8667 ! Valency undercoordination
6.0891 ! Valency angle/lone pair
1.0563 ! Valency angle 1
2.0384 ! Valency angle 2
6.1431 ! Not used 2
6.9290 ! Double bond/angle
0.3989 ! Double bond/angle: overcoord 1
3.9954 ! Double bond/angle: overcoord 2
-2.4837 ! Not used 3
5.7796 ! Torsion/BO
10.0000 ! Torsion overcoordination 1
1.9487 ! Torsion overcoordination 2
-1.2327 ! Not used 4
2.1645 ! Conjugation
1.5591 ! vdWaals shielding
0.1000 ! bond order cutoff
2.1365 ! Valency angle conjugation 2
0.6991 ! Valency overcoordination 1
50.0000 ! Valency overcoordination 2
1.8512 ! Valency/lone pair
0.5000 ! Not used 5
20.0000 ! Not used 6
5.0000 ! Not used 7
0.0000 ! Not used 8
2.6962 ! Valency angle conjugation 3
7 ! Nr of atoms; cov.r; valency;a.m;Rvdw;Evdw;gammaEEM;cov.r2;#
alfa;gammavdW;valency;Eunder;Eover;chiEEM;etaEEM;n.u.
cov r3;Elp;Heat inc.;n.u.;n.u.;n.u.;n.u.
ov/un;val1;n.u.;val3,vval4
C 1.3817 4.0000 12.0000 1.8903 0.1838 0.9000 1.1341 4.0000
9.7559 2.1346 4.0000 34.9350 79.5548 5.9666 7.0000 0.0000
1.2114 0.0000 202.5551 8.9539 34.9289 13.5366 0.8563 0.0000
-2.8983 2.5000 1.0564 4.0000 2.9663 0.0000 0.0000 0.0000
H 0.8930 1.0000 1.0080 1.3550 0.0930 0.8203 -0.1000 1.0000
8.2230 33.2894 1.0000 0.0000 121.1250 3.7248 9.6093 1.0000
-0.1000 0.0000 61.6606 3.0408 2.4197 0.0003 1.0698 0.0000
-19.4571 4.2733 1.0338 1.0000 2.8793 0.0000 0.0000 0.0000
O 1.2450 2.0000 15.9990 2.3890 0.1000 1.0898 1.0548 6.0000
9.7300 13.8449 4.0000 37.5000 116.0768 8.5000 8.3122 2.0000
0.9049 0.4056 59.0626 3.5027 0.7640 0.0021 0.9745 0.0000
-3.5500 2.9000 1.0493 4.0000 2.9225 0.0000 0.0000 0.0000
Fe 1.9029 3.0000 55.8450 2.0990 0.1181 0.4744 -1.6836 3.0000
10.8548 2.6084 3.0000 0.0000 18.3725 1.7785 8.6281 0.0000
-1.2000 0.0000 102.1000 25.3430 10.1260 0.7590 0.8563 0.0000
-16.0573 2.6997 1.0338 6.0000 2.5791 0.0000 0.0000 0.0000
S 1.8328 2.0000 32.0600 1.8815 0.3236 0.7530 1.6468 6.0000
9.0000 4.9055 4.0000 30.0000 112.1416 6.5745 9.0000 2.0000
1.0000 3.4994 65.0000 12.0000 22.1978 15.3230 0.9745 0.0000
-15.7363 2.8802 1.0338 6.2998 2.8793 0.0000 0.0000 0.0000
Cr 1.8921 6.0000 51.9962 2.3712 0.2336 0.5639 -1.6836 6.0000
10.3177 2.8702 6.0000 0.0000 18.3190 1.4546 8.9500 0.0000
-1.2000 0.0000 102.1000 25.3430 10.1260 0.7590 0.8563 0.0000
-11.1953 2.6997 1.0338 6.0000 2.5791 0.0000 0.0000 0.0000
X -0.1000 2.0000 1.0080 2.0000 0.0000 0.0100 -0.1000 6.0000
10.0000 2.5000 4.0000 0.0000 0.0000 5.0001 9999.0000 0.0000
-0.1000 0.0000 -2.3700 8.7410 13.3640 0.6690 0.9745 0.0000
-11.0000 2.7466 1.0338 2.0000 2.8793 0.0000 0.0000 0.0000
21 ! Nr of bonds; Edis1;LPpen;n.u.;pbe1;pbo5;13corr;pbo6
pbe2;pbo3;pbo4;n.u.;pbo1;pbo2;ovcorr
1 1 158.2004 99.1897 78.0000 -0.7738 -0.4550 1.0000 37.6117 0.4147
0.4590 -0.1000 9.1628 1.0000 -0.0777 6.7268 1.0000 0.0000
1 2 169.4760 0.0000 0.0000 -0.6083 0.0000 1.0000 6.0000 0.7652
5.2290 1.0000 0.0000 1.0000 -0.0500 6.9136 0.0000 0.0000
2 2 153.3934 0.0000 0.0000 -0.4600 0.0000 1.0000 6.0000 0.7300
6.2500 1.0000 0.0000 1.0000 -0.0790 6.0552 0.0000 0.0000
1 3 164.4303 82.6772 60.8077 -0.3739 -0.2351 1.0000 10.5036 1.0000
0.4475 -0.2288 7.0250 1.0000 -0.1363 4.8734 0.0000 0.0000
2 3 160.0000 0.0000 0.0000 -0.5725 0.0000 1.0000 6.0000 0.5626
1.1150 1.0000 0.0000 0.0000 -0.0920 4.2790 0.0000 0.0000
3 3 142.2858 145.0000 50.8293 0.2506 -0.1000 1.0000 29.7503 0.6051
0.3451 -0.1055 9.0000 1.0000 -0.1225 5.5000 1.0000 0.0000
1 4 109.5214 0.0000 0.0000 0.6663 -0.3000 1.0000 36.0000 0.0100
1.0648 -0.3500 15.0000 1.0000 -0.1512 4.1708 1.0000 0.0000
2 4 78.2669 0.0000 0.0000 0.4668 0.0000 1.0000 6.0000 0.1766
0.5673 1.0000 0.0000 1.0000 -0.1543 5.4965 0.0000 0.0000
3 4 67.5128 0.0000 0.0000 0.1301 -0.3000 0.0000 36.0000 0.0852
1.0000 -0.3500 15.0000 1.0000 -0.0629 7.1208 0.0000 0.0000
4 4 41.4611 0.0000 0.0000 0.2931 -0.2000 0.0000 16.0000 0.2682
0.6294 -0.2000 15.0000 1.0000 -0.0512 6.8013 0.0000 0.0000
1 5 192.1462 90.5383 55.2528 -0.5652 -0.5211 1.0000 18.9617 0.1958
2.0000 -0.1016 13.8750 1.0000 -0.1579 5.5813 1.0000 0.0000
2 5 188.3744 0.0000 0.0000 -0.6562 0.0000 1.0000 6.0000 0.3870
11.8360 1.0000 0.0000 1.0000 -0.0762 5.0961 1.0000 0.0000
3 5 107.2917 202.9813 40.0000 0.4728 -0.2406 1.0000 22.1005 0.0500
0.6528 -0.3341 7.9877 1.0000 -0.0909 6.9512 1.0000 0.0000
4 5 75.5280 0.0000 0.0000 -0.4815 -0.3390 0.0000 16.0000 0.1769
0.2800 -0.1838 15.0000 1.0000 -0.0758 6.3424 0.0000 0.0000
5 5 86.8868 69.1367 0.0000 -0.9993 -0.4781 1.0000 17.8574 0.0999
0.2799 -0.1677 8.2557 1.0000 -0.1131 6.1440 1.0000 0.0000
2 6 0.0000 0.0000 0.0000 1.0000 -0.3000 1.0000 36.0000 0.5000
0.3000 -0.3500 15.0000 1.0000 -0.2000 8.0000 1.0000 0.0000
3 6 114.0666 0.0000 0.0000 0.2305 -0.3000 1.0000 36.0000 0.6591
0.5793 -0.3500 15.0000 1.0000 -0.1989 4.8803 1.0000 0.0000
5 6 88.6258 0.0000 0.0000 0.6879 -0.3891 1.0000 36.0000 0.0835
0.2900 -0.2339 12.1279 1.0000 -0.1166 5.6660 1.0000 0.0000
6 6 57.5947 0.0000 0.0000 -1.0000 -0.3000 0.0000 16.0000 0.0100
0.0842 -0.3000 16.0000 1.0000 -0.1098 5.3349 0.0000 0.0000
4 6 41.4444 0.0000 0.0000 0.9374 -0.1000 0.0000 16.0000 0.0106
0.2209 -0.2000 15.0000 1.0000 -0.0925 6.1220 0.0000 0.0000
1 6 0.0000 0.0000 0.0000 1.0000 -0.3000 1.0000 36.0000 0.5000
0.3000 -0.3500 15.0000 1.0000 -0.2000 8.0000 1.0000 0.0000
15 ! Nr of off-diagonal terms; Ediss;Ro;gamma;rsigma;rpi;rpi2
1 2 0.1239 1.4004 9.8467 1.1210 -1.0000 -1.0000
1 3 0.1345 1.8422 9.7725 1.2835 1.1576 1.0637
2 3 0.0283 1.2885 10.9190 0.9215 -1.0000 -1.0000
1 4 0.4204 1.4900 11.0144 1.4071 -1.0000 -1.0000
2 4 0.0200 1.9451 10.8595 1.4157 -1.0000 -1.0000
3 4 0.1000 1.8000 9.1989 1.7050 -1.0000 -1.0000
1 5 0.3314 1.7976 10.5605 1.6918 1.4000 -1.0000
2 5 0.1020 1.7528 9.6276 1.3714 -1.0000 -1.0000
3 5 0.2832 1.8196 10.2295 1.4502 1.4557 -1.0000
4 5 0.0854 1.7474 12.7924 1.9838 -1.0000 -1.0000
2 6 0.1000 1.5000 11.0000 -1.0000 -1.0000 -1.0000
3 6 0.0582 1.7000 11.6513 1.5924 -1.0000 -1.0000
5 6 0.2523 1.9844 10.1396 1.8938 -1.0000 -1.0000
4 6 0.3236 2.1670 10.1297 1.9025 -1.0000 -1.0000
1 6 0.1000 1.5000 11.0000 -1.0000 -1.0000 -1.0000
120 ! Nr of angles;at1;at2;at3;Thetao,o;ka;kb;pv1;pv2
1 1 1 59.0573 30.7029 0.7606 0.0000 0.7180 6.2933 1.1244
1 1 2 65.7758 14.5234 6.2481 0.0000 0.5665 0.0000 1.6255
2 1 2 70.2607 25.2202 3.7312 0.0000 0.0050 0.0000 2.7500
1 2 1 0.0000 3.4110 7.7350 0.0000 0.0000 0.0000 1.0400
1 2 2 0.0000 0.0000 6.0000 0.0000 0.0000 0.0000 1.0400
2 2 2 0.0000 27.9213 5.8635 0.0000 0.0000 0.0000 1.0400
1 1 3 53.9517 7.8968 2.6122 0.0000 3.0000 58.6562 1.0338
3 1 3 76.9627 44.2852 2.4177 -25.3063 1.6334 -50.0000 2.7392
1 3 1 72.6199 42.5510 0.7205 0.0000 2.9294 0.0000 1.3096
1 3 3 81.9029 32.2258 1.7397 0.0000 0.9888 68.1072 1.7777
3 3 3 80.7324 30.4554 0.9953 0.0000 3.0000 50.0000 1.0783
2 1 3 65.0000 16.3141 5.2730 0.0000 0.4448 0.0000 1.4077
1 3 2 70.1101 13.1217 4.4734 0.0000 0.8433 0.0000 3.0000
2 3 3 75.6935 50.0000 2.0000 0.0000 1.0000 0.0000 1.1680
2 3 2 85.8000 9.8453 2.2720 0.0000 2.8635 0.0000 1.5800
1 2 3 0.0000 25.0000 3.0000 0.0000 1.0000 0.0000 1.0400
2 2 3 0.0000 8.5744 3.0000 0.0000 0.0000 0.0000 1.0421
3 2 3 0.0000 15.0000 2.8900 0.0000 0.0000 0.0000 2.8774
1 4 1 0.1000 42.2980 0.3169 0.0000 1.1069 0.0000 2.3466
1 1 4 74.8790 30.0000 2.0000 0.0000 2.0334 0.0000 1.0928
1 4 4 47.9341 1.0246 7.9341 0.0000 2.8853 0.0000 1.0000
4 1 4 33.2812 34.6443 3.0111 0.0000 0.1701 0.0000 1.0510
2 4 2 20.3683 0.0100 2.2825 0.0000 0.7660 0.0000 1.3788
2 2 4 0.0000 0.0100 1.0568 0.0000 1.8595 0.0000 3.6142
4 2 4 0.0000 10.4428 7.9607 0.0000 2.3717 0.0000 1.1970
2 4 4 48.4128 4.0632 0.6773 0.0000 2.2274 0.0000 1.8605
2 1 4 2.6539 32.1638 0.9167 0.0000 0.0240 0.0000 1.1158
1 4 2 42.7140 0.1451 0.2500 0.0000 0.0851 0.0000 2.8955
1 2 4 0.0000 0.0100 2.2066 0.0000 1.9789 0.0000 1.4466
1 3 4 90.0000 42.4716 6.6776 0.0000 2.4560 0.0000 1.6221
3 1 4 54.6900 12.6123 2.3543 0.0000 2.0000 0.0000 1.2513
1 4 3 38.2755 19.3103 0.1151 0.0000 0.7569 0.0000 2.3113
2 3 4 26.0012 49.6772 0.0500 0.0000 1.1589 0.0000 1.0000
3 2 4 0.0000 0.0100 3.2567 0.0000 2.0582 0.0000 1.3513
2 4 3 38.5594 11.2599 0.1898 0.0000 0.1904 0.0000 1.4041
4 3 4 63.0740 14.8127 2.9929 0.0000 0.7552 0.0000 1.3634
3 3 4 73.6721 32.6330 1.7223 0.0000 1.0221 0.0000 1.4351
3 4 3 76.5431 0.0583 0.0500 0.0000 0.4968 0.0000 2.2792
3 4 4 69.4895 5.7742 8.0001 0.0000 1.7794 0.0000 2.7889
5 5 5 84.2345 15.5790 3.7715 0.0000 1.3066 0.0000 1.6270
2 5 2 90.0601 42.2756 0.5302 0.0000 0.3707 0.0000 1.0071
2 5 5 66.1035 8.0885 1.0424 0.0000 0.7355 0.0000 3.0000
1 1 5 71.4462 27.2223 6.7228 0.0000 0.0050 0.0000 2.6454
1 5 1 92.6710 15.6798 3.1104 0.0000 0.3458 0.0000 2.3207
2 1 5 43.6595 11.8933 0.5449 0.0000 0.0050 0.0000 1.9326
1 5 2 99.1897 14.1666 2.5588 0.0000 0.3542 0.0000 2.5990
1 5 5 89.3910 5.0000 7.0000 0.0000 1.0050 0.0000 1.5000
3 5 3 83.5231 37.5859 0.9881 -0.0100 1.4725 0.0000 1.0641
1 5 3 79.9791 29.5117 7.0000 0.0000 0.0050 0.0000 1.2255
1 3 5 83.1032 23.4174 0.7741 0.0000 1.2168 0.0000 2.7365
3 3 5 60.2631 30.0701 2.1707 0.0000 1.3323 0.0000 1.0192
3 5 5 55.9402 38.2990 3.6930 0.0000 2.2673 0.0000 1.0000
2 3 5 35.9099 19.1501 1.9918 0.0000 0.9342 0.0000 2.7883
1 2 5 0.0000 0.2500 6.0000 0.0000 0.0000 0.0000 1.0400
3 2 5 0.0000 1.0000 6.0000 0.0000 0.0000 0.0000 1.0400
5 2 5 0.0000 7.5000 2.0000 0.0000 0.0000 0.0000 1.0400
5 4 5 0.2000 7.5083 1.3736 0.0000 0.0412 0.0000 1.8149
4 5 4 100.0000 9.2519 0.7752 0.0000 0.1221 0.0000 2.2142
4 4 5 33.4003 16.6274 0.1076 0.0000 0.0825 0.0000 1.0000
4 5 5 77.0475 7.4569 4.9579 0.0000 0.7548 0.0000 2.3345
2 5 4 84.8837 16.4865 0.8240 0.5000 0.5428 0.0000 1.1398
4 2 5 0.0000 10.0000 1.0000 0.5000 0.2500 0.0000 1.5000
3 4 5 29.3282 8.1452 2.8121 0.0000 0.2287 0.0000 1.9412
4 3 5 4.8461 5.5458 2.9681 0.0000 1.0491 0.0000 1.4836
3 5 4 89.9073 7.0973 1.3919 0.0000 0.8210 0.0000 2.9921
3 6 3 63.9878 12.9742 3.0000 0.0000 0.5058 0.0000 3.0000
6 3 6 54.9212 26.4957 1.1780 0.0000 2.0000 0.0000 2.2398
3 3 6 44.9499 30.0000 4.2281 0.0000 1.6672 0.0000 1.2000
3 6 6 51.2416 17.2148 0.3698 0.0000 1.5585 0.0000 1.5755
2 3 6 70.0000 20.0000 2.0000 0.0000 1.0000 0.0000 1.0500
2 6 3 0.0000 0.0000 1.0000 0.0000 1.0000 0.0000 1.0500
3 2 6 0.0000 0.0000 1.0000 0.0000 1.0000 0.0000 1.0500
2 6 6 0.0000 0.0000 1.0000 0.0000 1.0000 0.0000 1.0500
2 6 2 70.0000 20.0000 2.0000 0.0000 1.0000 0.0000 1.0500
6 2 6 0.0000 0.0000 1.0000 0.0000 1.0000 0.0000 1.0500
2 2 6 70.0000 20.0000 2.0000 0.0000 1.0000 0.0000 1.0500
5 6 5 70.0000 20.0000 2.0000 0.0000 1.0000 0.0000 1.0000
6 5 6 87.7418 23.5125 2.9950 0.0000 0.5199 0.0000 1.6571
5 5 6 70.0000 20.0000 2.0000 0.0000 1.0000 0.0000 1.0000
5 6 6 50.5809 19.2597 0.3584 0.0000 0.2153 0.0000 1.6345
2 5 6 0.0000 0.0000 1.0000 0.0000 1.0000 0.0000 1.0500
1 5 6 0.0000 0.0000 1.0000 0.0000 1.0000 0.0000 1.0500
5 3 5 76.3546 40.0000 3.3161 0.0005 1.9473 0.0000 1.0000
4 3 6 61.0461 0.2600 1.3988 0.0000 0.9535 0.0000 3.0000
3 6 4 44.3862 26.6099 2.0221 0.0000 1.9212 0.0000 3.0000
3 4 6 41.9243 20.0000 2.9881 0.0000 1.2678 0.0000 2.6481
3 6 5 19.4072 10.4590 2.8285 0.0000 1.0074 0.0000 2.3949
3 5 6 83.6415 19.7500 2.9247 0.0000 1.9437 0.0000 1.0348
5 3 6 89.6617 15.3276 2.7567 0.0000 1.6031 0.0000 1.0781
5 4 6 0.2000 7.5083 1.3736 0.0000 0.0412 0.0000 1.8149
4 6 5 70.0000 20.0000 2.0000 0.0000 1.0000 0.0000 1.0000
4 5 6 93.8709 16.3822 1.8851 0.0000 0.3210 0.0000 1.9357
1 4 5 60.3275 30.0000 1.0871 0.0000 1.3106 0.0000 1.6977
1 5 4 67.5806 21.4421 0.9049 0.0000 0.7789 0.0000 1.2000
4 1 5 89.6657 30.0000 3.0000 0.0000 2.0000 0.0000 1.2000
1 2 6 0.0000 0.0000 1.0000 0.0000 1.0000 0.0000 1.0500
1 6 2 70.0000 20.0000 2.0000 0.0000 1.0000 0.0000 1.0500
2 1 6 70.0000 20.0000 2.0000 0.0000 1.0000 0.0000 1.0500
1 6 6 0.0000 0.0000 1.0000 0.0000 1.0000 0.0000 1.0500
1 6 1 70.0000 20.0000 2.0000 0.0000 1.0000 0.0000 1.0500
6 1 6 0.0000 0.0000 1.0000 0.0000 1.0000 0.0000 1.0500
1 1 6 70.0000 20.0000 2.0000 0.0000 1.0000 0.0000 1.0500
1 6 3 70.0000 20.0000 2.0000 0.0000 1.0000 0.0000 1.0500
3 1 6 70.0000 20.0000 2.0000 0.0000 1.0000 0.0000 1.0500
1 3 6 63.7706 34.5234 0.9493 0.0000 2.4647 0.0000 1.7747
1 6 4 0.0000 0.0000 1.0000 0.0000 1.0000 0.0000 1.0500
1 4 6 0.0000 0.0000 1.0000 0.0000 1.0000 0.0000 1.0500
4 1 6 0.0000 0.0000 1.0000 0.0000 1.0000 0.0000 1.0500
2 6 4 0.0000 0.0000 1.0000 0.0000 1.0000 0.0000 1.0500
2 4 6 0.0000 0.0000 1.0000 0.0000 1.0000 0.0000 1.0500
4 2 6 0.0000 0.0000 1.0000 0.0000 1.0000 0.0000 1.0500
1 2 6 0.0000 0.0000 1.0000 0.0000 1.0000 0.0000 1.0500
1 6 2 0.0000 0.0000 1.0000 0.0000 1.0000 0.0000 1.0500
2 1 6 0.0000 0.0000 1.0000 0.0000 1.0000 0.0000 1.0500
1 5 6 70.0000 20.0000 2.0000 0.0000 1.0000 0.0000 1.0500
1 6 5 70.0000 20.0000 2.0000 0.0000 1.0000 0.0000 1.0500
5 1 6 0.0000 0.0000 1.0000 0.0000 1.0000 0.0000 1.0500
2 6 5 70.0000 20.0000 2.0000 0.0000 1.0000 0.0000 1.0500
2 5 6 70.0000 20.0000 2.0000 0.0000 1.0000 0.0000 1.0500
5 2 6 0.0000 0.0000 1.0000 0.0000 1.0000 0.0000 1.0500
38 ! Nr of torsions;at1;at2;at3;at4;;V1;V2;V3;V2(BO);vconj;n.u;n
1 1 1 1 -0.2500 34.7453 0.0288 -6.3507 -1.6000 0.0000 0.0000
1 1 1 2 -0.2500 29.2131 0.2945 -4.9581 -2.1802 0.0000 0.0000
2 1 1 2 -0.2500 31.2081 0.4539 -4.8923 -2.2677 0.0000 0.0000
1 1 1 3 1.2799 20.7787 -0.5249 -2.5000 -1.0000 0.0000 0.0000
2 1 1 3 1.9159 19.8113 0.7914 -4.6995 -1.0000 0.0000 0.0000
3 1 1 3 -1.4477 16.6853 0.6461 -4.9622 -1.0000 0.0000 0.0000
1 1 3 1 0.4816 19.6316 -0.0057 -2.5000 -1.0000 0.0000 0.0000
1 1 3 2 1.2044 80.0000 -0.3139 -6.1481 -1.0000 0.0000 0.0000
2 1 3 1 -2.5000 31.0191 0.6165 -2.7733 -2.9807 0.0000 0.0000
2 1 3 2 -2.4875 70.8145 0.7582 -4.2274 -3.0000 0.0000 0.0000
2 1 3 3 -1.4383 80.0000 1.0000 -3.6877 -2.8000 0.0000 0.0000
3 1 3 1 -1.1390 78.0747 -0.0964 -4.5172 -3.0000 0.0000 0.0000
3 1 3 2 -2.5000 70.3345 -1.0000 -5.5315 -3.0000 0.0000 0.0000
3 1 3 3 -0.1583 20.0000 1.5000 -9.0000 -2.0000 0.0000 0.0000
1 3 3 1 0.0002 80.0000 -1.5000 -4.4848 -2.0000 0.0000 0.0000
1 3 3 2 -2.5000 0.1181 0.0268 -5.4085 -2.9498 0.0000 0.0000
2 3 3 2 0.1995 5.0000 0.2000 -2.6000 0.0000 0.0000 0.0000
1 3 3 3 0.4118 0.5219 0.9706 -2.5004 -0.9972 0.0000 0.0000
2 3 3 3 0.1000 43.1840 0.5000 -6.6539 0.0000 0.0000 0.0000
3 3 3 3 0.1000 1.0000 0.1000 -2.5000 -0.9000 0.0000 0.0000
0 1 2 0 0.0000 0.0000 0.0000 0.0000 0.0000 0.0000 0.0000
0 2 2 0 0.0000 0.0000 0.0000 0.0000 0.0000 0.0000 0.0000
0 2 3 0 0.0000 0.1000 0.0200 -2.5415 0.0000 0.0000 0.0000
0 1 1 0 0.0000 50.0000 0.3000 -4.0000 0.0000 0.0000 0.0000
0 3 3 0 0.5511 25.4150 1.1330 -5.1903 -1.0000 0.0000 0.0000
1 1 3 3 -0.0002 20.1851 0.1601 -9.0000 -2.0000 0.0000 0.0000
2 1 4 4 0.0000 0.0000 0.0000 -5.0000 0.0000 0.0000 0.0000
0 2 5 0 0.0000 0.0000 0.0000 0.0000 0.0000 0.0000 0.0000
5 5 5 5 2.4661 71.9719 0.0100 -8.0000 0.0000 0.0000 0.0000
2 5 5 2 0.0000 50.0000 0.0000 -8.0000 0.0000 0.0000 0.0000
2 3 5 3 2.5000 2.5000 0.2237 -10.0000 0.0000 0.0000 0.0000
0 1 5 0 0.5000 50.0000 0.5000 -10.0000 0.0000 0.0000 0.0000
0 3 5 0 0.5000 50.0000 0.5000 -8.0000 0.0000 0.0000 0.0000
3 5 5 5 0.2500 90.0000 0.5000 -6.0000 0.0000 0.0000 0.0000
3 5 5 3 0.2500 90.0000 0.5000 -6.0000 0.0000 0.0000 0.0000
1 5 5 1 0.0000 50.0000 0.0000 -8.0000 0.0000 0.0000 0.0000
1 5 5 2 0.0000 50.0000 0.0000 -8.0000 0.0000 0.0000 0.0000
2 5 5 2 0.0000 50.0000 0.0000 -8.0000 0.0000 0.0000 0.0000
4 ! Nr of hydrogen bonds;at1;at2;at3;Rhb;Dehb;vhb1
3 2 3 2.1200 -3.5800 1.4500 19.5000
3 2 5 2.5000 -1.0000 1.4500 19.5000
5 2 3 2.5000 -1.0000 1.4500 19.5000
5 2 5 2.5000 -2.0000 1.4500 19.5000
"""

    assert write_lammps(data) == expected2
